package org.processmining.logprojection;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Map;

import org.deckfour.xes.extension.XExtension;
import org.deckfour.xes.factory.XFactory;
import org.deckfour.xes.model.XAttribute;
import org.deckfour.xes.model.XAttributeBoolean;
import org.deckfour.xes.model.XAttributeContinuous;
import org.deckfour.xes.model.XAttributeDiscrete;
import org.deckfour.xes.model.XAttributeLiteral;
import org.deckfour.xes.model.XAttributeTimestamp;
import org.processmining.logprojection.DataManager.StringHandler;

public class Attribute implements Comparable<Attribute>, Serializable {

	protected static final String E_PREF = "E: ";
	protected static final String T_PREF = "T: ";

	private static final long serialVersionUID = -6323807930862393362L;

	public static final byte CONTINUOUS_FLAG = 0x01;

	public static final byte TIME_REL = 1;
	public static final byte CONT = 3;
	public static final byte TIME_ABS = 5;

	public static final byte LITR = 0;
	public static final byte BOOL = 2;
	public static final byte DISC = 4;

	protected final char[] key;
	protected final byte type;
	protected final long defaultValue;

	protected final boolean isEvent;

	protected String prefix;

	public Attribute(char[] key, byte type, long defaultValue, boolean isEvent) {
		this.key = key;
		this.type = type;
		this.defaultValue = defaultValue;
		this.isEvent = isEvent;
		this.prefix = isEvent ? E_PREF : T_PREF;
	}

	public Attribute(String key, byte type, long defaultValue, boolean isEvent) {
		this.key = key.toCharArray();
		this.type = type;
		this.defaultValue = defaultValue;
		this.isEvent = isEvent;
		this.prefix = isEvent ? E_PREF : T_PREF;
	}

	public Attribute(XAttribute attribute, long defaultValue, boolean isEvent) {
		this.key = attribute.getKey().toCharArray();
		this.defaultValue = defaultValue;
		this.isEvent = isEvent;
		this.prefix = isEvent ? E_PREF : T_PREF;

		if (attribute instanceof XAttributeBoolean) {
			type = BOOL;
		} else if (attribute instanceof XAttributeContinuous) {
			type = CONT;
		} else if (attribute instanceof XAttributeDiscrete) {
			type = DISC;
		} else if (attribute instanceof XAttributeTimestamp) {
			type = TIME_ABS;
		} else if (attribute instanceof XAttributeLiteral) {
			type = LITR;
		} else {
			throw new RuntimeException("Unsupported attribute type");
		}
	}

	public boolean equals(Object o) {
		if (o instanceof Attribute) {
			Attribute a = (Attribute) o;
			// return a.type == type && Arrays.equals(a.key, key);
			return (a.isEvent == isEvent) && Arrays.equals(a.key, key)
					&& a.prefix.equals(prefix);
		}
		return false;
	}

	public int hashCode() {
		// return Arrays.hashCode(key) + 37 * type;
		return Arrays.hashCode(key) * (isEvent ? 37 : 1) + prefix.hashCode();
	}

	public String getKey() {
		return new String(key);
	}

	public byte getType() {
		return type;
	}

	public String toString() {
		return prefix + getKey();
	}

	public long getDefaultValue() {
		return defaultValue;
	}

	public static int sizeForType(byte type) {
		if (type == BOOL) {
			return 1;
		} else if (type == LITR) {
			return 4;
		} else {
			return 8;
		}
	}

	public static String toString(byte type) {
		switch (type) {
		case BOOL:
			return "boolean";
		case LITR:
			return "literal";
		case CONT:
			return "continuous";
		case DISC:
			return "discrete";
		case TIME_ABS:
			return "timestamp";
		case TIME_REL:
			return "timesince";
		default:
			return "unknown";
		}
	}

	@Override
	public int compareTo(Attribute o) {
		int r = prefix.compareTo(o.prefix);
		if (r != 0) {
			return r;
		}
		for (int i = 0; i < key.length && i < o.key.length; i++) {
			if (key[i] != o.key[i]) {
				return key[i] - o.key[i];
			}
		}
		if (key.length != o.key.length) {
			return key.length - o.key.length;
		}
		return 0;
	}

	public XAttribute toXAttribute(XFactory factory,
			Map<String, XExtension> key2extension, StringHandler handler) {
		String skey = getKey();
		switch (type) {
		case BOOL:
			return factory.createAttributeBoolean(skey, defaultValue == 1,
					key2extension.get(skey));
		case LITR:
			return factory.createAttributeLiteral(skey,
					handler.getLiteral((int) defaultValue),
					key2extension.get(skey));
		case CONT:
			return factory.createAttributeContinuous(skey,
					Double.longBitsToDouble(defaultValue),
					key2extension.get(skey));
		case DISC:
			return factory.createAttributeDiscrete(skey, defaultValue,
					key2extension.get(skey));
		case TIME_ABS:
		case TIME_REL:
			return factory.createAttributeTimestamp(skey, defaultValue,
					key2extension.get(skey));
		default:
			return null;
		}
	}

	public boolean isEventAttribute() {
		return isEvent;
	}
}