package org.processmining.logprojection.abstraction;

import gnu.trove.map.TObjectIntMap;
import gnu.trove.map.hash.TObjectIntHashMap;
import gnu.trove.procedure.TObjectIntProcedure;
import gnu.trove.procedure.TObjectProcedure;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import org.processmining.logprojection.lightxlog.LightLog;
import org.processmining.logprojection.lightxlog.LightTrace;

public class MultiPhaseMinerMatrix implements CausalDependencyMatrix {

	private static final int FIRST = -1;
	private static final int LAST = -2;
	private static final String FIRST_STRING = "Start";
	private static final String LAST_STRING = "End";
	private static final int NONE = -3;

	private static final double SIMILARITY_THRESHOLD = 0.1;
	private static final int DEPTH = 2;
	private static final double CAUSALTHRESHOLD = 0.7;
	private static final boolean USEFREQUENCIES = false;

	private static class FindRepresentativesProcedure implements
			TObjectIntProcedure<MultiPhaseContext> {

		private final MultiPhaseContext key;
		private List<MultiPhaseContext> representatives = new ArrayList<MultiPhaseContext>(
				5);
		private final double similarityThreshold;

		public FindRepresentativesProcedure(MultiPhaseContext key,
				double similarityThreshold) {
			this.key = key;
			this.similarityThreshold = similarityThreshold;
		}

		@Override
		public boolean execute(MultiPhaseContext a, int b) {
			double sim = key.getSimilarity(a);
			if (sim >= similarityThreshold) {
				representatives.add(a);
				// } else if (sim > 0) {
				// System.out.println(a + " & " + key + " = " + sim);
			}
			return true;
		}

		public List<MultiPhaseContext> getRepresentatives() {
			return representatives;
		}
	};

	private static class FindRepresentativeProcedure implements
			TObjectIntProcedure<MultiPhaseContext> {

		private final MultiPhaseContext key;
		private MultiPhaseContext representative;
		private double bestFound = 0;

		public FindRepresentativeProcedure(MultiPhaseContext key) {
			this.key = key;
		}

		@Override
		public boolean execute(MultiPhaseContext a, int b) {
			double sim = key.getSimilarity(a);
			if (sim > bestFound) {
				representative = a;
				bestFound = sim;
			}
			return true;
		}

		public MultiPhaseContext getRepresentative() {
			return representative;
		}
	};

	protected final int[][] predecessors;
	protected final int[][] successors;

	protected final int[][] edgeOccCount;
	protected final int[] occCount;
	protected final int[] preSetSize;
	protected final int[] postSetSize;

	protected final LightLog log;
	private final TObjectIntMap<String> name2eventIndex;
	private CausalDependencyMatrix causal;

	public MultiPhaseMinerMatrix(LightLog log, CausalDependencyMatrix causal) {
		this.log = log;
		this.name2eventIndex = new TObjectIntHashMap<String>(log
				.getName2EventIndex().size(), 0.7f, LightLog.NONAME);
		log.getName2EventIndex().forEachEntry(
				new TObjectIntProcedure<String>() {

					@Override
					public boolean execute(String a, int b) {
						name2eventIndex.put(a, b);
						return true;
					}
				});
		name2eventIndex.put(FIRST_STRING, FIRST);
		name2eventIndex.put(LAST_STRING, LAST);

		this.causal = causal;
		this.predecessors = new int[log.size()][];
		this.successors = new int[log.size()][];

		for (int t = 0; t < log.size(); t++) {
			buildTrace(t);
		}

		reLabelLog(causal);

		this.edgeOccCount = new int[log.getNumEventClasses() + 2][log
				.getNumEventClasses() + 2];
		this.occCount = new int[log.getNumEventClasses() + 2];
		this.preSetSize = new int[log.getNumEventClasses() + 2];
		this.postSetSize = new int[log.getNumEventClasses() + 2];

		computeOccurenceCounts();
	}

	protected void reLabelLog(CausalDependencyMatrix matrix) {
		int nextLabel = matrix.getNumberOfNodes();

		int max = matrix.getNumberOfNodes();
		for (int i = 0; i < max; i++) {
			if (matrix.getOccurenceCount(i) == 0) {
				continue;
			}
			TObjectIntMap<MultiPhaseContext> eventContexts = getFeatureSet(i,
					USEFREQUENCIES);
			if (eventContexts.size() <= 1) {
				continue;
			}
			final TObjectIntMap<MultiPhaseContext> relabelMap = new TObjectIntHashMap<MultiPhaseContext>();
			final int labelStart = nextLabel;
			eventContexts.forEachKey(new TObjectProcedure<MultiPhaseContext>() {
				private boolean first = true;
				private int l = 0;

				@Override
				public boolean execute(MultiPhaseContext object) {
					if (first) {
						first = false;
					} else {
						relabelMap.put(object, labelStart + l);
						l++;
					}
					return true;
				}
			});
			nextLabel += eventContexts.size() - 1;

			// event type i has multiple contexts. Let's relabel this event.
			for (int t = 0; t < log.size(); t++) {
				LightTrace trace = log.getTrace(t);
				for (int e = 0; e < trace.size(); e++) {
					if (trace.getEvent(e).getNameIndex() + 1 != i) {
						continue;
					}
					// identify the context of the event and
					// "relabel" the event. this is currently not possible, but
					// should be made possible!
					MultiPhaseContext eventContext = getContextForEvent(t, e,
							DEPTH, USEFREQUENCIES);
					FindRepresentativeProcedure procedure = new FindRepresentativeProcedure(
							eventContext);
					eventContexts.forEachEntry(procedure);
					assert procedure.getRepresentative() != null;
					eventContext = procedure.getRepresentative();
					// relabel event e to eventContext;
					int label = relabelMap.get(eventContext);
					if (label > 0) {
						trace.getEvent(e).setNameIndex(label - 1);
					}
				}
			}
		}
		log.invalidate();
	}

	protected void buildTrace(int t) {
		LightTrace trace = log.getTrace(t);

		successors[t] = new int[trace.size()];
		Arrays.fill(successors[t], NONE);
		predecessors[t] = new int[trace.size()];
		Arrays.fill(predecessors[t], NONE);

		for (int e = 0; e < trace.size(); e++) {
			int evt = trace.getEvent(e).getNameIndex() + 1;

			int pre = getFirstPredecessor(trace, e, evt);
			predecessors[t][e] = pre;

			int suc = getFirstSuccessor(trace, e, evt);
			successors[t][e] = suc;
		}

	}

	private void computeOccurenceCounts() {
		for (int t = 0; t < predecessors.length; t++) {
			int[] preCount = new int[predecessors[t].length + 2];
			int[] postCount = new int[predecessors[t].length + 2];

			occCount[0]++;
			occCount[occCount.length - 1]++;
			for (int e = 0; e < predecessors[t].length; e++) {
				int evt = log.getTrace(t).getEvent(e).getNameIndex() + 1;
				occCount[evt]++;
				int from = predecessors[t][e];
				if (from == FIRST) {
					// edge from first to e
					edgeOccCount[0][evt]++;
					preCount[e + 1]++;
					postCount[0]++;
				} else if (from != NONE) {
					// edge from "from" to e
					preCount[e + 1]++;
					postCount[from + 1]++;
					edgeOccCount[log.getTrace(t).getEvent(from).getNameIndex() + 1][evt]++;

				}

				int to = successors[t][e];
				if (to == LAST) {
					edgeOccCount[evt][edgeOccCount.length - 1]++;
					preCount[preCount.length - 1]++;
					postCount[e + 1]++;
				} else if (to != NONE) {
					if (predecessors[t][to] != e) {
						preCount[to + 1]++;
						postCount[e + 1]++;
						// count only if e is not a predecessor of to (avoid
						// double count)
						edgeOccCount[evt][log.getTrace(t).getEvent(to)
								.getNameIndex() + 1]++;

					}
				}
			}
			for (int i = 0; i < preCount.length; i++) {
				int evt;
				if (i == 0) {
					evt = 0;
				} else if (i == preCount.length - 1) {
					evt = edgeOccCount.length - 1;
				} else {
					evt = log.getTrace(t).getEvent(i - 1).getNameIndex() + 1;
				}
				if (preCount[i] > preSetSize[evt]) {
					preSetSize[evt] = preCount[i];
				}
				if (postCount[i] > postSetSize[evt]) {
					postSetSize[evt] = postCount[i];
				}
			}
		}
		preSetSize[0] = 1;
		postSetSize[postSetSize.length - 1] = 1;
		System.out.println(Arrays.toString(preSetSize));
		System.out.println(Arrays.toString(postSetSize));
	}

	protected MultiPhaseContext getContextForEvent(int t, int e, int depth,
			boolean useFrequencies) {
		LightTrace trace = log.getTrace(t);
		int evt = trace.getEvent(e).getNameIndex() + 1;
		MultiPhaseContext c = new MultiPhaseContext(evt, useFrequencies,
				name2eventIndex);
		addPredecessorsToContext(c, trace, t, e, 0, depth);
		addSuccessorsToContext(c, trace, t, e, 0, depth);
		return c;
	}

	public TObjectIntMap<MultiPhaseContext> getFeatureSet(int node,
			final boolean useFrequencies) {

		TObjectIntMap<MultiPhaseContext> features = new TObjectIntHashMap<MultiPhaseContext>();

		for (int t = 0; t < log.size(); t++) {
			LightTrace trace = log.getTrace(t);

			for (int e = 0; e < trace.size(); e++) {
				int evt = trace.getEvent(e).getNameIndex() + 1;
				if (evt == node) {
					MultiPhaseContext c = getContextForEvent(t, e, DEPTH,
							useFrequencies);
					features.adjustOrPutValue(c, 1, 1);
				}
			}
		}

		final TObjectIntMap<MultiPhaseContext> uniqueFeatures = new TObjectIntHashMap<MultiPhaseContext>();

		features.forEachEntry(new TObjectIntProcedure<MultiPhaseContext>() {

			@Override
			public boolean execute(MultiPhaseContext key, int value) {
				// consider the set of unique features. If the current feature
				// is disjoint, add it. If it is not disjoint, replace the
				// existing one if this has a higher frequency.
				FindRepresentativesProcedure procedure = new FindRepresentativesProcedure(
						key, SIMILARITY_THRESHOLD);
				uniqueFeatures.forEachEntry(procedure);
				List<MultiPhaseContext> representatives = procedure
						.getRepresentatives();

				final MultiPhaseContext representative = new MultiPhaseContext(
						key.getEvent(), useFrequencies, name2eventIndex);

				int repVal = value;
				for (MultiPhaseContext c : representatives) {
					repVal += uniqueFeatures.get(c);
					uniqueFeatures.remove(c);
					representative.addToPre(c.getPreSet());
					representative.addToPost(c.getPostSet());
				}

				representative.addToPre(key.getPreSet());
				representative.addToPost(key.getPostSet());

				uniqueFeatures.put(representative, repVal);
				return true;
			}
		});

		return uniqueFeatures;
	}

	private void addPredecessorsToContext(MultiPhaseContext c,
			LightTrace trace, int t, int e, int depth, int maxDepth) {
		if (predecessors[t][e] == FIRST) {
			c.addToPre(FIRST_STRING);
			// there cannot be any other predecessors
		} else if (predecessors[t][e] == NONE) {
			// there are no predecessors
			return;
		} else {
			c.addToPre(trace.getEvent(predecessors[t][e]).getName());

			if (depth < maxDepth) {
				addPredecessorsToContext(c, trace, t, predecessors[t][e],
						depth + 1, maxDepth);
			}

			// find any preceding event of whom e is a successor
			for (int e2 = predecessors[t][e]; e2-- > 0;) {
				if (successors[t][e2] == e && e2 != predecessors[t][e]) {
					c.addToPre(trace.getEvent(e2).getName());

					if (depth < maxDepth) {
						addPredecessorsToContext(c, trace, t, e2, depth + 1,
								maxDepth);
					}

					// break;
				}
			}
		}
	}

	private void addSuccessorsToContext(MultiPhaseContext c, LightTrace trace,
			int t, int e, int depth, int maxDepth) {
		if (successors[t][e] == LAST) {
			c.addToPost(LAST_STRING);
			// there cannot be any other predecessors
		} else if (successors[t][e] == NONE) {
			// there are no successors
			return;
		} else {
			c.addToPost(trace.getEvent(successors[t][e]).getName());

			if (depth < maxDepth) {
				addSuccessorsToContext(c, trace, t, successors[t][e],
						depth + 1, maxDepth);
			}

			// find any preceding event of whom e is a successor
			for (int e2 = successors[t][e] + 1; e2 < trace.size(); e2++) {
				if (successors[t][e2] == e && e2 != successors[t][e]) {
					c.addToPost(trace.getEvent(e2).getName());

					if (depth < maxDepth) {
						addSuccessorsToContext(c, trace, t, e2, depth + 1,
								maxDepth);
					}

					// break;
				}
			}
		}
	}

	protected int getFirstPredecessor(LightTrace trace, int e, int evt) {

		double maxCausal = CAUSALTHRESHOLD;
		int pre = FIRST;
		for (int p = e; p-- > 0;) {
			double c = causal.getCausalDependency(trace.getEvent(p)
					.getNameIndex() + 1, evt);
			if (c > maxCausal) {
				maxCausal = c;
				pre = p;
				return pre;
			}
		}
		// check for the causal relation between event evt
		// the start of a trace

		if (causal.getCausalDependency(0, evt) > CAUSALTHRESHOLD) {
			return FIRST;
		} else {
			return NONE;
		}
	}

	protected int getFirstSuccessor(LightTrace trace, int e, int evt) {

		double maxCausal = CAUSALTHRESHOLD;
		int suc = LAST;
		for (int p = e + 1; p < trace.size(); p++) {
			double c = causal.getCausalDependency(evt, trace.getEvent(p)
					.getNameIndex() + 1);
			if (c > maxCausal) {
				maxCausal = c;
				suc = p;
				return suc;
			}
		}
		if (causal.getCausalDependency(evt, causal.getNumberOfNodes() - 1) > CAUSALTHRESHOLD) {
			return LAST;
		} else {
			return NONE;
		}
	}

	public int getSuccessionCount(int node1, int node2) {
		return edgeOccCount[node1][node2];
	}

	@Override
	public double getCausalDependency(int node1, int node2) {
		// return edgeOccCount[node1][node2] / (double) log.size();
		return edgeOccCount[node1][node2]
				/ (double) Math.max(getOccurenceCount(node1),
						getOccurenceCount(node2));
		// return edgeOccCount[node1][node2] / (double)
		// getOccurenceCount(node1);
	}

	@Override
	public int getNumberOfNodes() {
		return log.getNumEventClasses() + 2;
	}

	@Override
	public int getOccurenceCount(int i) {
		return occCount[i];
	}

	@Override
	public String getEventClass(int i) {
		if (i == 0) {
			return "start";
		} else if (i == occCount.length - 1) {
			return "end";
		}
		return log.getInt2EventClass()[i - 1];
	}

	public String toString() {
		return "MultiPhase miner";
	}

	@Override
	public int getCount(int node1, int node2) {
		return edgeOccCount[node1][node2];
	}

}
