package org.processmining.logprojection.plugins.dottedchart;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Paint;
import java.awt.Shape;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.text.NumberFormat;
import java.util.Iterator;
import java.util.List;

import org.jfree.chart.axis.AxisState;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTick;
import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.axis.Tick;
import org.jfree.chart.axis.TickUnit;
import org.jfree.chart.axis.TickUnitSource;
import org.jfree.chart.axis.ValueAxis;
import org.jfree.chart.axis.ValueTick;
import org.jfree.chart.event.AxisChangeEvent;
import org.jfree.chart.plot.Plot;
import org.jfree.chart.plot.PlotRenderingInfo;
import org.jfree.chart.plot.ValueAxisPlot;
import org.jfree.chart.util.ParamChecks;
import org.jfree.data.Range;
import org.jfree.text.TextUtilities;
import org.jfree.ui.RectangleEdge;
import org.jfree.ui.TextAnchor;
import org.processmining.logprojection.DataManager;

/**
 * A standard linear value axis that replaces integer values with symbols.
 */
public class LiteralAxis extends NumberAxis {

	/** For serialization. */
	private static final long serialVersionUID = 7216330468770619716L;

	/** The default grid band paint. */
	public static final Paint DEFAULT_GRID_BAND_PAINT = new Color(232, 234,
			232, 128);

	/**
	 * The default paint for alternate grid bands.
	 * 
	 * @since 1.0.7
	 */
	public static final Paint DEFAULT_GRID_BAND_ALTERNATE_PAINT = new Color(0,
			0, 0, 0); // transparent

	/** Flag that indicates whether or not grid bands are visible. */
	private boolean gridBandsVisible;

	/** The paint used to color the grid bands (if the bands are visible). */
	private transient Paint gridBandPaint;

	/**
	 * The paint used to fill the alternate grid bands.
	 * 
	 * @since 1.0.7
	 */
	private transient Paint gridBandAlternatePaint;

	private final DataManager<?> manager;

	private int[] literalIndices;

	/**
	 * Constructs a symbol axis, using default attribute values where necessary.
	 * 
	 * @param label
	 *            the axis label (<code>null</code> permitted).
	 * @param sv
	 *            the list of symbols to display instead of the numeric values.
	 */
	public LiteralAxis(String label, DataManager<?> manager,
			int[] literalIndices) {
		super(label);
		this.manager = manager;
		this.literalIndices = literalIndices;

		this.gridBandsVisible = true;
		this.gridBandPaint = DEFAULT_GRID_BAND_PAINT;
		this.gridBandAlternatePaint = DEFAULT_GRID_BAND_ALTERNATE_PAINT;
		setAutoTickUnitSelection(true, false);
		setAutoRangeStickyZero(true);

	}

	/**
	 * Returns <code>true</code> if the grid bands are showing, and
	 * <code>false</code> otherwise.
	 * 
	 * @return <code>true</code> if the grid bands are showing, and
	 *         <code>false</code> otherwise.
	 * 
	 * @see #setGridBandsVisible(boolean)
	 */
	public boolean isGridBandsVisible() {
		return this.gridBandsVisible;
	}

	/**
	 * Sets the visibility of the grid bands and notifies registered listeners
	 * that the axis has been modified.
	 * 
	 * @param flag
	 *            the new setting.
	 * 
	 * @see #isGridBandsVisible()
	 */
	public void setGridBandsVisible(boolean flag) {
		this.gridBandsVisible = flag;
		fireChangeEvent();
	}

	/**
	 * Returns the paint used to color the grid bands.
	 * 
	 * @return The grid band paint (never <code>null</code>).
	 * 
	 * @see #setGridBandPaint(Paint)
	 * @see #isGridBandsVisible()
	 */
	public Paint getGridBandPaint() {
		return this.gridBandPaint;
	}

	/**
	 * Sets the grid band paint and sends an {@link AxisChangeEvent} to all
	 * registered listeners.
	 * 
	 * @param paint
	 *            the paint (<code>null</code> not permitted).
	 * 
	 * @see #getGridBandPaint()
	 */
	public void setGridBandPaint(Paint paint) {
		ParamChecks.nullNotPermitted(paint, "paint");
		this.gridBandPaint = paint;
		fireChangeEvent();
	}

	/**
	 * Returns the paint used for alternate grid bands.
	 * 
	 * @return The paint (never <code>null</code>).
	 * 
	 * @see #setGridBandAlternatePaint(Paint)
	 * @see #getGridBandPaint()
	 * 
	 * @since 1.0.7
	 */
	public Paint getGridBandAlternatePaint() {
		return this.gridBandAlternatePaint;
	}

	/**
	 * Sets the paint used for alternate grid bands and sends a
	 * {@link AxisChangeEvent} to all registered listeners.
	 * 
	 * @param paint
	 *            the paint (<code>null</code> not permitted).
	 * 
	 * @see #getGridBandAlternatePaint()
	 * @see #setGridBandPaint(Paint)
	 * 
	 * @since 1.0.7
	 */
	public void setGridBandAlternatePaint(Paint paint) {
		ParamChecks.nullNotPermitted(paint, "paint");
		this.gridBandAlternatePaint = paint;
		fireChangeEvent();
	}

	/**
	 * This operation is not supported by this axis.
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param dataArea
	 *            the area in which the plot and axes should be drawn.
	 * @param edge
	 *            the edge along which the axis is drawn.
	 */
	@Override
	protected void selectAutoTickUnit(Graphics2D g2, Rectangle2D dataArea,
			RectangleEdge edge) {
		if (RectangleEdge.isTopOrBottom(edge)) {
			selectHorizontalAutoTickUnit(g2, dataArea, edge);
		} else if (RectangleEdge.isLeftOrRight(edge)) {
			selectVerticalAutoTickUnit(g2, dataArea, edge);
		}

	}

	/**
	 * Selects an appropriate tick value for the axis. The strategy is to
	 * display as many ticks as possible (selected from an array of 'standard'
	 * tick units) without the labels overlapping.
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param dataArea
	 *            the area defined by the axes.
	 * @param edge
	 *            the axis location.
	 */
	protected void selectHorizontalAutoTickUnit(Graphics2D g2,
			Rectangle2D dataArea, RectangleEdge edge) {

		double tickLabelWidth = estimateMaximumTickLabelWidth(g2, getTickUnit());

		// start with the current tick unit...
		TickUnitSource tickUnits = getStandardTickUnits();
		TickUnit unit1 = tickUnits.getCeilingTickUnit(getTickUnit());
		double unit1Width = lengthToJava2D(unit1.getSize(), dataArea, edge);

		// then extrapolate...
		double guess = (tickLabelWidth / unit1Width) * unit1.getSize();
		if (guess < 1) {
			guess = 1;
		}
		NumberTickUnit unit2 = (NumberTickUnit) tickUnits
				.getCeilingTickUnit(guess);
		double unit2Width = lengthToJava2D(unit2.getSize(), dataArea, edge);

		tickLabelWidth = estimateMaximumTickLabelWidth(g2, unit2);
		if (tickLabelWidth > unit2Width) {
			unit2 = (NumberTickUnit) tickUnits.getLargerTickUnit(unit2);
		}

		setTickUnit(unit2, false, false);

	}

	/**
	 * Selects an appropriate tick value for the axis. The strategy is to
	 * display as many ticks as possible (selected from an array of 'standard'
	 * tick units) without the labels overlapping.
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param dataArea
	 *            the area in which the plot should be drawn.
	 * @param edge
	 *            the axis location.
	 */
	protected void selectVerticalAutoTickUnit(Graphics2D g2,
			Rectangle2D dataArea, RectangleEdge edge) {

		double tickLabelHeight = estimateMaximumTickLabelHeight(g2);

		// start with the current tick unit...
		TickUnitSource tickUnits = getStandardTickUnits();
		TickUnit unit1 = tickUnits.getCeilingTickUnit(getTickUnit());
		double unitHeight = lengthToJava2D(unit1.getSize(), dataArea, edge);
		double guess = unit1.getSize();
		if (unitHeight > 0) {
			// then extrapolate...
			guess = (tickLabelHeight / unitHeight) * unit1.getSize();
		}
		if (guess < 1) {
			guess = 1;
		}
		NumberTickUnit unit2 = (NumberTickUnit) tickUnits
				.getCeilingTickUnit(guess);
		double unit2Height = lengthToJava2D(unit2.getSize(), dataArea, edge);

		tickLabelHeight = estimateMaximumTickLabelHeight(g2);
		if (tickLabelHeight > unit2Height) {
			unit2 = (NumberTickUnit) tickUnits.getLargerTickUnit(unit2);
		}

		setTickUnit(unit2, false, false);

	}

	/**
	 * Draws the axis on a Java 2D graphics device (such as the screen or a
	 * printer).
	 * 
	 * @param g2
	 *            the graphics device (<code>null</code> not permitted).
	 * @param cursor
	 *            the cursor location.
	 * @param plotArea
	 *            the area within which the plot and axes should be drawn (
	 *            <code>null</code> not permitted).
	 * @param dataArea
	 *            the area within which the data should be drawn (
	 *            <code>null</code> not permitted).
	 * @param edge
	 *            the axis location (<code>null</code> not permitted).
	 * @param plotState
	 *            collects information about the plot (<code>null</code>
	 *            permitted).
	 * 
	 * @return The axis state (never <code>null</code>).
	 */
	@Override
	public AxisState draw(Graphics2D g2, double cursor, Rectangle2D plotArea,
			Rectangle2D dataArea, RectangleEdge edge,
			PlotRenderingInfo plotState) {

		AxisState info = new AxisState(cursor);
		if (isVisible()) {
			info = super.draw(g2, cursor, plotArea, dataArea, edge, plotState);
		}
		if (this.gridBandsVisible) {
			drawGridBands(g2, plotArea, dataArea, edge, info.getTicks());
		}
		return info;

	}

	/**
	 * Draws the grid bands. Alternate bands are colored using
	 * <CODE>gridBandPaint</CODE> (<CODE>DEFAULT_GRID_BAND_PAINT</CODE> by
	 * default).
	 * 
	 * @param g2
	 *            the graphics target (<code>null</code> not permitted).
	 * @param plotArea
	 *            the area within which the plot is drawn (<code>null</code> not
	 *            permitted).
	 * @param dataArea
	 *            the data area to which the axes are aligned (<code>null</code>
	 *            not permitted).
	 * @param edge
	 *            the edge to which the axis is aligned (<code>null</code> not
	 *            permitted).
	 * @param ticks
	 *            the ticks (<code>null</code> not permitted).
	 */
	@SuppressWarnings("rawtypes")
	protected void drawGridBands(Graphics2D g2, Rectangle2D plotArea,
			Rectangle2D dataArea, RectangleEdge edge, List ticks) {
		Shape savedClip = g2.getClip();
		g2.clip(dataArea);
		if (RectangleEdge.isTopOrBottom(edge)) {
			drawGridBandsHorizontal(g2, plotArea, dataArea, true, ticks);
		} else if (RectangleEdge.isLeftOrRight(edge)) {
			drawGridBandsVertical(g2, plotArea, dataArea, true, ticks);
		}
		g2.setClip(savedClip);
	}

	/**
	 * Draws the grid bands for the axis when it is at the top or bottom of the
	 * plot.
	 * 
	 * @param g2
	 *            the graphics target (<code>null</code> not permitted).
	 * @param plotArea
	 *            the area within which the plot is drawn (not used here).
	 * @param dataArea
	 *            the area for the data (to which the axes are aligned,
	 *            <code>null</code> not permitted).
	 * @param firstGridBandIsDark
	 *            True: the first grid band takes the color of
	 *            <CODE>gridBandPaint</CODE>. False: the second grid band takes
	 *            the color of <CODE>gridBandPaint</CODE>.
	 * @param ticks
	 *            a list of ticks (<code>null</code> not permitted).
	 */
	@SuppressWarnings("rawtypes")
	protected void drawGridBandsHorizontal(Graphics2D g2, Rectangle2D plotArea,
			Rectangle2D dataArea, boolean firstGridBandIsDark, List ticks) {

		boolean currentGridBandIsDark = firstGridBandIsDark;
		double yy = dataArea.getY();
		double xx1, xx2;

		// gets the outline stroke width of the plot
		double outlineStrokeWidth = 1.0;
		Stroke outlineStroke = getPlot().getOutlineStroke();
		if (outlineStroke != null && outlineStroke instanceof BasicStroke) {
			outlineStrokeWidth = ((BasicStroke) outlineStroke).getLineWidth();
		}

		Iterator iterator = ticks.iterator();
		ValueTick tick;
		Rectangle2D band;
		while (iterator.hasNext()) {
			tick = (ValueTick) iterator.next();
			xx1 = valueToJava2D(tick.getValue() - 0.5d, dataArea,
					RectangleEdge.BOTTOM);
			xx2 = valueToJava2D(tick.getValue() + 0.5d, dataArea,
					RectangleEdge.BOTTOM);
			if (currentGridBandIsDark) {
				g2.setPaint(this.gridBandPaint);
			} else {
				g2.setPaint(this.gridBandAlternatePaint);
			}
			band = new Rectangle2D.Double(Math.min(xx1, xx2), yy
					+ outlineStrokeWidth, Math.abs(xx2 - xx1),
					dataArea.getMaxY() - yy - outlineStrokeWidth);
			g2.fill(band);
			currentGridBandIsDark = !currentGridBandIsDark;
		}
	}

	/**
	 * Draws the grid bands for an axis that is aligned to the left or right of
	 * the data area (that is, a vertical axis).
	 * 
	 * @param g2
	 *            the graphics target (<code>null</code> not permitted).
	 * @param plotArea
	 *            the area within which the plot is drawn (not used here).
	 * @param dataArea
	 *            the area for the data (to which the axes are aligned,
	 *            <code>null</code> not permitted).
	 * @param firstGridBandIsDark
	 *            True: the first grid band takes the color of
	 *            <CODE>gridBandPaint</CODE>. False: the second grid band takes
	 *            the color of <CODE>gridBandPaint</CODE>.
	 * @param ticks
	 *            a list of ticks (<code>null</code> not permitted).
	 */
	@SuppressWarnings("rawtypes")
	protected void drawGridBandsVertical(Graphics2D g2, Rectangle2D plotArea,
			Rectangle2D dataArea, boolean firstGridBandIsDark, List ticks) {

		boolean currentGridBandIsDark = firstGridBandIsDark;
		double xx = dataArea.getX();
		double yy1, yy2;

		// gets the outline stroke width of the plot
		double outlineStrokeWidth = 1.0;
		Stroke outlineStroke = getPlot().getOutlineStroke();
		if (outlineStroke != null && outlineStroke instanceof BasicStroke) {
			outlineStrokeWidth = ((BasicStroke) outlineStroke).getLineWidth();
		}

		Iterator iterator = ticks.iterator();
		ValueTick tick;
		Rectangle2D band;
		while (iterator.hasNext()) {
			tick = (ValueTick) iterator.next();
			yy1 = valueToJava2D(tick.getValue() + 0.5d, dataArea,
					RectangleEdge.LEFT);
			yy2 = valueToJava2D(tick.getValue() - 0.5d, dataArea,
					RectangleEdge.LEFT);
			if (currentGridBandIsDark) {
				g2.setPaint(this.gridBandPaint);
			} else {
				g2.setPaint(this.gridBandAlternatePaint);
			}
			band = new Rectangle2D.Double(xx + outlineStrokeWidth, Math.min(
					yy1, yy2), dataArea.getMaxX() - xx - outlineStrokeWidth,
					Math.abs(yy2 - yy1));
			g2.fill(band);
			currentGridBandIsDark = !currentGridBandIsDark;
		}
	}

	/**
	 * Rescales the axis to ensure that all data is visible.
	 */
	@Override
	protected void autoAdjustRange() {
		Plot plot = getPlot();
		if (plot == null) {
			return; // no plot, no data
		}

		if (plot instanceof ValueAxisPlot) {

			// ensure that all the symbols are displayed
			double upper = this.literalIndices.length;
			double lower = 0;
			double range = upper - lower;

			// ensure the autorange is at least <minRange> in size...
			double minRange = getAutoRangeMinimumSize();
			if (range < minRange) {
				upper = (upper + lower + minRange) / 2;
				lower = (upper + lower - minRange) / 2;
			}

			// this ensure that the grid bands will be displayed correctly.
			double upperMargin = getUpperMargin();
			double lowerMargin = getLowerMargin();

			if (getAutoRangeIncludesZero()) {
				if (getAutoRangeStickyZero()) {
					if (upper <= 0.0) {
						upper = 0.0;
					} else {
						upper = upper + upperMargin * range;
					}
					if (lower >= 0.0) {
						lower = 0.0;
					} else {
						lower = lower - lowerMargin * range;
					}
				} else {
					upper = Math.max(0.0, upper + upperMargin * range);
					lower = Math.min(0.0, lower - lowerMargin * range);
				}
			} else {
				if (getAutoRangeStickyZero()) {
					if (upper <= 0.0) {
						upper = Math.min(0.0, upper + upperMargin * range);
					} else {
						upper = upper + upperMargin * range;
					}
					if (lower >= 0.0) {
						lower = Math.max(0.0, lower - lowerMargin * range);
					} else {
						lower = lower - lowerMargin * range;
					}
				} else {
					upper = upper + upperMargin * range;
					lower = lower - lowerMargin * range;
				}
			}
			setRange(new Range(lower, upper), false, false);
		}
	}

	/**
	 * Calculates the positions of the tick labels for the axis, storing the
	 * results in the tick label list (ready for drawing).
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param state
	 *            the axis state.
	 * @param dataArea
	 *            the area in which the data should be drawn.
	 * @param edge
	 *            the location of the axis.
	 * 
	 * @return A list of ticks.
	 */
	@Override
	@SuppressWarnings("rawtypes")
	public List refreshTicks(Graphics2D g2, AxisState state,
			Rectangle2D dataArea, RectangleEdge edge) {
		List ticks = null;
		if (RectangleEdge.isTopOrBottom(edge)) {
			ticks = refreshTicksHorizontal(g2, dataArea, edge);
		} else if (RectangleEdge.isLeftOrRight(edge)) {
			ticks = refreshTicksVertical(g2, dataArea, edge);
		}
		return ticks;
	}

	/**
	 * Calculates the positions of the tick labels for the axis, storing the
	 * results in the tick label list (ready for drawing).
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param dataArea
	 *            the area in which the data should be drawn.
	 * @param edge
	 *            the location of the axis.
	 * 
	 * @return The ticks.
	 */
	@Override
	@SuppressWarnings({ "rawtypes", "unchecked" })
	protected List refreshTicksHorizontal(Graphics2D g2, Rectangle2D dataArea,
			RectangleEdge edge) {

		List ticks = new java.util.ArrayList();

		Font tickLabelFont = getTickLabelFont();
		g2.setFont(tickLabelFont);

		if (isAutoTickUnitSelection()) {
			selectAutoTickUnit(g2, dataArea, edge);
		}

		double size = getTickUnit().getSize();
		int count = calculateVisibleTickCount();
		double lowestTickValue = calculateLowestVisibleTickValue();

		double previousDrawnTickLabelPos = 0.0;
		double previousDrawnTickLabelLength = 0.0;

		if (count <= ValueAxis.MAXIMUM_TICK_COUNT) {
			for (int i = 0; i < count; i++) {
				double currentTickValue = lowestTickValue + (i * size);
				double xx = valueToJava2D(currentTickValue, dataArea, edge);
				String tickLabel;
				NumberFormat formatter = getNumberFormatOverride();
				if (formatter != null) {
					tickLabel = formatter.format(currentTickValue);
				} else {
					tickLabel = valueToString(currentTickValue);
				}

				// avoid to draw overlapping tick labels
				Rectangle2D bounds = TextUtilities.getTextBounds(tickLabel, g2,
						g2.getFontMetrics());
				double tickLabelLength = isVerticalTickLabels() ? bounds
						.getHeight() : bounds.getWidth();
				boolean tickLabelsOverlapping = false;
				if (i > 0) {
					double avgTickLabelLength = (previousDrawnTickLabelLength + tickLabelLength) / 2.0;
					if (Math.abs(xx - previousDrawnTickLabelPos) < avgTickLabelLength) {
						tickLabelsOverlapping = true;
					}
				}
				if (tickLabelsOverlapping) {
					tickLabel = ""; // don't draw this tick label
				} else {
					// remember these values for next comparison
					previousDrawnTickLabelPos = xx;
					previousDrawnTickLabelLength = tickLabelLength;
				}

				TextAnchor anchor;
				TextAnchor rotationAnchor;
				double angle = 0.0;
				if (isVerticalTickLabels()) {
					anchor = TextAnchor.CENTER_RIGHT;
					rotationAnchor = TextAnchor.CENTER_RIGHT;
					if (edge == RectangleEdge.TOP) {
						angle = Math.PI / 2.0;
					} else {
						angle = -Math.PI / 2.0;
					}
				} else {
					if (edge == RectangleEdge.TOP) {
						anchor = TextAnchor.BOTTOM_CENTER;
						rotationAnchor = TextAnchor.BOTTOM_CENTER;
					} else {
						anchor = TextAnchor.TOP_CENTER;
						rotationAnchor = TextAnchor.TOP_CENTER;
					}
				}
				Tick tick = new NumberTick(new Double(currentTickValue),
						tickLabel, anchor, rotationAnchor, angle);
				ticks.add(tick);
			}
		}
		return ticks;

	}

	/**
	 * Calculates the positions of the tick labels for the axis, storing the
	 * results in the tick label list (ready for drawing).
	 * 
	 * @param g2
	 *            the graphics device.
	 * @param dataArea
	 *            the area in which the plot should be drawn.
	 * @param edge
	 *            the location of the axis.
	 * 
	 * @return The ticks.
	 */
	@Override
	@SuppressWarnings({ "rawtypes", "unchecked" })
	protected List refreshTicksVertical(Graphics2D g2, Rectangle2D dataArea,
			RectangleEdge edge) {

		List ticks = new java.util.ArrayList();

		Font tickLabelFont = getTickLabelFont();
		g2.setFont(tickLabelFont);

		if (isAutoTickUnitSelection()) {
			selectAutoTickUnit(g2, dataArea, edge);
		}

		double size = getTickUnit().getSize();
		int count = calculateVisibleTickCount();
		double lowestTickValue = calculateLowestVisibleTickValue();

		double previousDrawnTickLabelPos = 0.0;
		double previousDrawnTickLabelLength = 0.0;

		if (count <= ValueAxis.MAXIMUM_TICK_COUNT) {
			for (int i = 0; i < count; i++) {
				double currentTickValue = lowestTickValue + (i * size);
				double yy = valueToJava2D(currentTickValue, dataArea, edge);
				String tickLabel;
				NumberFormat formatter = getNumberFormatOverride();
				if (formatter != null) {
					tickLabel = formatter.format(currentTickValue);
				} else {
					tickLabel = valueToString(currentTickValue);
				}

				// avoid to draw overlapping tick labels
				Rectangle2D bounds = TextUtilities.getTextBounds(tickLabel, g2,
						g2.getFontMetrics());
				double tickLabelLength = isVerticalTickLabels() ? bounds
						.getWidth() : bounds.getHeight();
				boolean tickLabelsOverlapping = false;
				if (i > 0) {
					double avgTickLabelLength = (previousDrawnTickLabelLength + tickLabelLength) / 2.0;
					if (Math.abs(yy - previousDrawnTickLabelPos) < avgTickLabelLength) {
						tickLabelsOverlapping = true;
					}
				}
				if (tickLabelsOverlapping) {
					tickLabel = ""; // don't draw this tick label
				} else {
					// remember these values for next comparison
					previousDrawnTickLabelPos = yy;
					previousDrawnTickLabelLength = tickLabelLength;
				}

				TextAnchor anchor;
				TextAnchor rotationAnchor;
				double angle = 0.0;
				if (isVerticalTickLabels()) {
					anchor = TextAnchor.BOTTOM_CENTER;
					rotationAnchor = TextAnchor.BOTTOM_CENTER;
					if (edge == RectangleEdge.LEFT) {
						angle = -Math.PI / 2.0;
					} else {
						angle = Math.PI / 2.0;
					}
				} else {
					if (edge == RectangleEdge.LEFT) {
						anchor = TextAnchor.CENTER_RIGHT;
						rotationAnchor = TextAnchor.CENTER_RIGHT;
					} else {
						anchor = TextAnchor.CENTER_LEFT;
						rotationAnchor = TextAnchor.CENTER_LEFT;
					}
				}
				Tick tick = new NumberTick(new Double(currentTickValue),
						tickLabel, anchor, rotationAnchor, angle);
				ticks.add(tick);
			}
		}
		return ticks;

	}

	/**
	 * Converts a value to a string, using the list of symbols.
	 * 
	 * @param value
	 *            value to convert.
	 * 
	 * @return The symbol.
	 */
	public String valueToString(double value) {

		if (value < 0 || value >= literalIndices.length) {
			return "";
		}

		String strToReturn;
		try {
			strToReturn = manager.getStringHandler().getLiteral(
					literalIndices[(int) value]);
		} catch (IndexOutOfBoundsException ex) {
			strToReturn = "";
		}
		return strToReturn;
	}

	public void setLiteralIndices(int[] literalIndices) {
		this.literalIndices = literalIndices;
	}

	/**
	 * Tests this axis for equality with an arbitrary object.
	 * 
	 * @param obj
	 *            the object (<code>null</code> permitted).
	 * 
	 * @return A boolean.
	 */
}
